@@ -3,15 +3,17 @@
 
 import argparse
 import codecs
+import inspect
 import locale
 import logging
 import logging.handlers
 import os
 import sys
+import operator
 
 from .complete import CompleteCommand
 from .help import HelpAction, HelpCommand
-from .interactive import InteractiveApp
+from .utils import damerau_levenshtein, COST
 
 # Make sure the cliff library has a logging handler
 # in case the app developer doesn't set up logging.
@@ -64,7 +66,7 @@ class App(object):
 
     def __init__(self, description, version, command_manager,
                  stdin=None, stdout=None, stderr=None,
-                 interactive_app_factory=InteractiveApp,
+                 interactive_app_factory=None,
                  deferred_help=False):
         """Initialize the application.
         """
@@ -79,7 +81,10 @@ class App(object):
         self.interpreter = None
 
     def _set_streams(self, stdin, stdout, stderr):
-        locale.setlocale(locale.LC_ALL, '')
+        try:
+            locale.setlocale(locale.LC_ALL, '')
+        except locale.Error:
+            pass
         if sys.version_info[:2] == (2, 6):
             # Configure the input and output streams. If a stream is
             # provided, it must be configured correctly by the
@@ -206,7 +211,7 @@ class App(object):
         """
         if self.deferred_help and self.options.deferred_help:
             action = HelpAction(None, None, default=self)
-            action(self.parser, self.parser, None, None)
+            action(self.parser, self.options, None, None)
 
     def run(self, argv):
         """Equivalent to the main program for the application.
@@ -218,6 +223,18 @@ class App(object):
             self.options, remainder = self.parser.parse_known_args(argv)
             self.configure_logging()
             self.interactive_mode = not remainder
+            if self.deferred_help and self.options.deferred_help and remainder:
+                # When help is requested and `remainder` has any values disable
+                # `deferred_help` and instead allow the help subcommand to
+                # handle the request during run_subcommand(). This turns
+                # "app foo bar --help" into "app help foo bar". However, when
+                # `remainder` is empty use print_help_if_requested() to allow
+                # for an early exit.
+                # Disabling `deferred_help` here also ensures that
+                # print_help_if_requested will not fire if called by a subclass
+                # during its initialize_app().
+                self.options.deferred_help = False
+                remainder.insert(0, "help")
             self.initialize_app(remainder)
             self.print_help_if_requested()
         except Exception as err:
@@ -271,6 +288,11 @@ class App(object):
         return
 
     def interact(self):
+        # Defer importing .interactive as cmd2 is a slow import
+        from .interactive import InteractiveApp
+
+        if self.interactive_app_factory is None:
+            self.interactive_app_factory = InteractiveApp
         self.interpreter = self.interactive_app_factory(self,
                                                         self.command_manager,
                                                         self.stdin,
@@ -279,17 +301,66 @@ class App(object):
         self.interpreter.cmdloop()
         return 0
 
+    def get_fuzzy_matches(self, cmd):
+        """return fuzzy matches of unknown command
+        """
+
+        sep = '_'
+        if self.command_manager.convert_underscores:
+            sep = ' '
+        all_cmds = [k[0] for k in self.command_manager]
+        dist = []
+        for candidate in sorted(all_cmds):
+            prefix = candidate.split(sep)[0]
+            # Give prefix match a very good score
+            if candidate.startswith(cmd):
+                dist.append((candidate, 0))
+                continue
+            # Levenshtein distance
+            dist.append((candidate, damerau_levenshtein(cmd, prefix, COST)+1))
+        dist = sorted(dist, key=operator.itemgetter(1, 0))
+        matches = []
+        i = 0
+        # Find the best similarity
+        while (not dist[i][1]):
+            matches.append(dist[i][0])
+            i += 1
+        best_similarity = dist[i][1]
+        while (dist[i][1] == best_similarity):
+            matches.append(dist[i][0])
+            i += 1
+
+        return matches
+
     def run_subcommand(self, argv):
         try:
             subcommand = self.command_manager.find_command(argv)
         except ValueError as err:
-            if self.options.debug:
-                raise
+            # If there was no exact match, try to find a fuzzy match
+            the_cmd = argv[0]
+            fuzzy_matches = self.get_fuzzy_matches(the_cmd)
+            if fuzzy_matches:
+                article = 'a'
+                if self.NAME[0] in 'aeiou':
+                    article = 'an'
+                self.stdout.write('%s: \'%s\' is not %s %s command. '
+                                  'See \'%s --help\'.\n'
+                                  % (self.NAME, the_cmd, article,
+                                      self.NAME, self.NAME))
+                self.stdout.write('Did you mean one of these?\n')
+                for match in fuzzy_matches:
+                    self.stdout.write('  %s\n' % match)
             else:
-                self.LOG.error(err)
+                if self.options.debug:
+                    raise
+                else:
+                    self.LOG.error(err)
             return 2
         cmd_factory, cmd_name, sub_argv = subcommand
-        cmd = cmd_factory(self, self.options)
+        kwargs = {}
+        if 'cmd_name' in inspect.getargspec(cmd_factory.__init__).args:
+            kwargs['cmd_name'] = cmd_name
+        cmd = cmd_factory(self, self.options, **kwargs)
         err = None
         result = 1
         try:
